; Purpose:	Solution to Experiment 4: Shifting the Display of the LCD unit
; 			Optrex C-51505NFJ-SLW-AIN
;			Employs Novatek's NT3881D Dot Matrix LCD Controller and Driver
; Author: 	C. D'Arcy
; Date:		10 01 08 
; Display connector description:
;  ==============================================================================================
;  |  1   |  2   |  3  |  4  |  5  |  6  |  7  |  8  |  9  | 10  | 11  | 12 | 13 | 14 | 15 | 16 | 
;  |=============================================================================================
;  |LED(A)|LED(K)| GND | VCC | VEE | RS  | R/W |  E  | D0  | D1  | D2  | D3 | D4 | D5 | D6 | D7 |
;  ==============================================================================================
;
;  Description:
;
;  LED(A) =  positive supply for the background light (if your display has one)
;  LED(K) =  negative supply for the background light (if your display has one)
;  GND    =  signal ground ( GND of Atmel board )
;  VCC    =  +5V supply voltage ( VTG on Atmel board )
;  VEE    =  voltage input for display contrast  ()
;  RS     =  register select input 
;  R/W    =  read/write signal input
;  E      =  enable signal input
;  D0-D7  =  data bits from D0 to D7 
;==========================================================================
;--------------------------------------------------------------------------
; I N C L U D E S
;--------------------------------------------------------------------------
.include	"m16def.inc"
.include	"LCDdef.inc"
;--------------------------------------------------------------------------
; D E F I N E S
;--------------------------------------------------------------------------
.def	temp	= R16					;
.def	temp2	= R17					;
.def	data	= R18					;
.def	cmd		= R19					;
.def	times	= R24					;
.def	delays	= R25					;
.def	res1	= R20					; Reserved for Delay Routines
.def	res2	= R21					; "
.def	res3	= R22					; "
.def	res4	= R23					; "
;--------------------------------------------------------------------------
; E Q U A T E S
;--------------------------------------------------------------------------
.equ	start	= 0x41					; First of the 80 chars loaded in DDRAM
.equ	end		= 0x91					; One past the last char loaded in DDRAM
.equ	columns = 20					; Number of columns to shift
.equ	DataDDR = DDRA					;
.equ	DataPort= PORTA					;
.equ	ControlDDR	= DDRB				;
.equ	ControlPort	= PORTB				;
;--------------------------------------------------------------------------
; I N T E R R U P T   V E C T O R   T A B L E
;--------------------------------------------------------------------------
.org	0x0000
		rjmp	RESET
;--------------------------------------------------------------------------
; L E T ' S   G O
;--------------------------------------------------------------------------
Reset:
		ldi		temp,high(RAMEND)		;set the Stack Pointer High (SPH) register
		out		SPH, temp				;to the high byte of RAM ($45F)
		ldi		temp,low(RAMEND)		;set the Stack Pointer Low (SPL) register
		out		SPL, temp				;to the low byte of RAM ($45F)

		ser		temp					;prepare for output on DDR PORT registers
		out		ControlDDR,temp			;declare all DDR bits for Control to be used for output
		out		DataDDR,temp			;declare all DDR bits for Data to be used for output

		rcall	LCDInit					;ready the LCD
		rcall	LCDLoad					;fill DDRAM with characters
Shift:									;alternate between shift display left, shift display right, and so on...
		rcall	Delay1s					;observe...
		ldi		cmd,SHIFTLEFT			;predefined equate in LCDdef.inc
		ldi		times,columns			;set the number of columns to shfit display by
Left:
		rcall 	LCDCommand				;execute command
		rcall	Delay250ms				;not too fast...
		dec		times					;one less column to shift
		brne	Left					;still more?
		rcall	Delay1s					;settle...
		ldi		cmd,SHIFTRIGHT			;predefined equate in LCDdef.inc
		ldi		times,columns			;reset column count
Right:
		rcall 	LCDCommand				;execute command to shift display right
		rcall	Delay250ms				;not too fast...
		dec		times					;one less column to shift
		brne	Right					;still more?	
		rjmp	Shift					;no, ok repeat
;----------------------------------------------------------------------------------------
; F I L L   D D R A M   W I T H   C H A R S 
;----------------------------------------------------------------------------------------
LCDLoad:								;Fill DDRAM with characters
		ldi		data,start				;block character
Next:
		rcall	LCDData
		inc		data
		cpi		data,end
		brne	Next
		ret
;----------------------------------------------------------------------------------------
; L C D    R O U T I N E S 
;----------------------------------------------------------------------------------------
LCDInit:
		ldi		delays,15				;wait 15ms after PowerUp
Try1:	   
		rcall	Delay1ms				
		dec		delays
		brne	Try1
		ldi		cmd,LCD_FS|(1<<LCD_dl)|(1<<LCD_n)	;Function Set
		rcall 	LCDCommand				;execute command
		ldi		delays,5				;wait 5ms
Try2:
		rcall	Delay1ms
		dec		delays
		brne	Try2
		rcall	Delay1ms				;wait 1ms

		ldi		cmd,LCD_DSP				;Display Off
		rcall	LCDCommand				;execute command
		ldi		cmd,LCD_CD				;Clear Display
		rcall	LCDCommand				;execute command
		ldi		cmd,LCD_EMS|(1<<LCD_id)	;Entry Mode Set
		rcall	LCDCommand				;execute command
		ldi		cmd,LCD_DSP|(1<<LCD_d)	;Display On
		rcall	LCDCommand				;execute command
		ret
LCDCommand:								;write command to LCD (command is in cmd)
		out 	DataPort, cmd			;put command on the bus
		ldi		temp,WRTCOM
		out		ControlPort,temp
		rcall	LCDWrite
		ret								;return from subroutine
LCDData:								;write data to LCD routine (data is in data)
		out 	DataPort, data			;put command on the bus
		ldi		temp,WRTDAT
		out		ControlPort,temp
		rcall	LCDWrite
		ret								;return from subroutine
LCDWrite:
		rcall	Delay1ms
		sbi 	ControlPort,LCD_E		;set Enable > writes command
		rcall	Delay1ms
		cbi		ControlPort,LCD_E		;
		rcall	Delay1ms
		ret								;return from subroutine
;---------------------------------------------------------------------------------------
; D E L A Y    R O U T I N E S
;---------------------------------------------------------------------------------------
Delay1s:
		ldi		temp2,4					;M16 can deliver faster that the LCD can receive/process 
PowerUpWait2:							; so delays are key to assist with synch
        rcall	Delay250ms
        dec		temp2
		brne	PowerUpWait2
		ret
	

Delay250ms:
		ldi		temp,255				;M16 can deliver faster that the LCD can receive/process 
PowerUpWait:							; so delays are key to assist with synch
        rcall	Delay1ms
        dec		temp
		brne	PowerUpWait
		ret

Delay100us:
; ============================= 
;    delay loop generator 
;     100 cycles:
; ----------------------------- 
; delaying 99 cycles:
          ldi  R20, $21
WGLOOP0:  dec  R20
          brne WGLOOP0
; ----------------------------- 
; delaying 1 cycle:
          nop
; ============================= 
		ret

Delay1ms:
; ============================= 
;    delay loop generator 
;     1000 cycles:
; ----------------------------- 
; delaying 999 cycles:
          ldi  R20, $03
WGLOOP10:  ldi  R21, $6E
WGLOOP11:  dec  R21
          brne WGLOOP11
          dec  R20
          brne WGLOOP10
; ----------------------------- 
; delaying 1 cycle:
          nop
; ============================= 
		ret
